/* 
==========================================================================================================================
| Author   :  Mike G7GTN - South Devon
| Email    :  mikeg7gtn@gmail.com
| Date     :  28th May 2015
| Version  :  V1.01a
| Filename :  PS2TYPE.ino
| Project  :  Simple TV Typewriter PS/2 Keyboard Driven
==========================================================================================================================

Hardware:	
           Ardunio Nano (16MHz)
	   VIDEO DAC 1K & 470R
           Standard PC compatible UK PS/2 Keyboard 			

User Keys:      
             
           ESC   - Clear Screen
           Home  - Set position 0,0 (Top Left)
           Enter - New Line
           Press & Hold Shift key to enter uppercase text
          	
I/O Usage:
                                                                           
           ARDUNIO DIGITAL I/O 7 VIDEO VIA 470R RESISTOR
           ARDUNIO DIGITAL I/O 9 VIDEO SYNC VIA 1K RESISTOR
           ARDUNIO DIGITAL I/O 0 (RX PIN) PS/2 KEYBOARD DATA PIN
           ARDUNIO DIGITAL I/O 4 PS/2 KEYBOARD CLOCK PIN
                                                                          
==========================================================================================================================
*/

const char Version [] = "1.01a";
#define Width 120			        
#define Height 50
//#define Debug

#include <TVout.h>            			// Include TVOUT Library
#include <fontALL.h>				
#include <PS2uartKeyboard.h>			// Non interrupt Based Keyboard Driver Via TV.set_hbi_hook

TVout TV;
PS2uartKeyboard keyboard;
char Text;                                      
int charcount;                                  // Characters in a line

//----------------------------------------------------------------------------------------------------
// General setup
//----------------------------------------------------------------------------------------------------
void setup()  
{  
  pinMode(13, OUTPUT);				// On board LED
  digitalWrite(13, LOW);
  TV.begin(PAL,Width, Height);                  // Start Video @ dimensions    
  TV.set_hbi_hook(keyboard.begin()); 		// Hook PS/2 keyboard in to TV Horizontal Blanking Period
  TV.select_font(font8x8);			// Use Large Font 8 X 8 Pixels
  TV.fill(0);					// Clear Screen
  #if defined Debug    
     TV.println(Version);
     delay(3000); 			         
     TV.fill(0);  
  #endif 
  TV.set_cursor(0,0);				// Text Starting Position, Top Left
  charcount=0;
}

//----------------------------------------------------------------------------------------------------
// Get users Keyboard input
//----------------------------------------------------------------------------------------------------
void getps2keys()
{
 if (keyboard.available())                      // Keyboard available?
 {			
     char c = keyboard.read();	        	// Read key value 
     Text = c; 
   
     #if defined Debug    
       TV.print(c, DEC);                       // Just print key code
     #endif

    if  (c != PS2_ENTER)
    { 
        TV.print(Text);
        charcount = charcount + 1;
        // See when video screen overflows
        if (charcount >= 84)
        {
          pinMode(13, OUTPUT);		       // On board LED
          digitalWrite(13, HIGH);
          charcount=0;                         // Reset character counter
          TV.set_cursor(0,0);
        }          
    }  
      
    // See if we can have a new line? 
    if(c == PS2_ENTER) 
    {
      if (charcount < 70)
      {
         charcount = charcount + 1;           // Add one
         TV.println(" ");
      }
      if (charcount >= 84)                    // No, reset to top
      {
         charcount=0;  
         TV.set_cursor(0,0);
      } 
    }

    if(c == PS2_ESC)                       // Clear Teletype Screen
    {	
      pinMode(13, OUTPUT);		   // On board LED
      digitalWrite(13, LOW);		      
      TV.fill(0);
    }

    if(c == PS2_HOME) 
    {	
      pinMode(13, OUTPUT);		   // On board LED
      digitalWrite(13, LOW);		
      TV.fill(0);                               
      TV.set_cursor(0,0);                  // Set Cursor Position to 0,0  
    }

    if(c == PS2_F1)                        // User Defineable 
    {			
      F1();
    }

    if(c == PS2_F2) 
    {			
      F2();					
    }

    if(c == PS2_F3) 
    {			
      F3();
    }

    if(c == PS2_F4) 
    {			
      F4();
    }

    if(c == PS2_F5) 
    {			
      F5();
    }

    if(c == PS2_F6) 
    {			
      F6();
    }

    if(c == PS2_F7) 
    {			
      F7();
    }

    if(c == PS2_F8) 
    {			
      F8();
    }

    if(c == PS2_F9) 
    {			
      F9();
    }

    if(c == PS2_F10) 
    {			
      F10();
    }

    if(c == PS2_F11) 
    {			
      F11();
    }

    if(c == PS2_F12) 
    {			
      F12();
    }  
    
    }           
}

//----------------------------------------------------------------------------------------------------
// Check the PS2uartKeyboard.h file for a list of all usable keys.
// You can also map any keys that you require.
//----------------------------------------------------------------------------------------------------

void F1()
{
   TV.fill(0);					 
   TV.println(2,2,"F1 Pressed");
}

void F2() 
{
   TV.fill(0);					
   TV.println(2,2,"F2 Pressed");		
}

void F3() 
{
   TV.fill(0);
   TV.println(2,2,"F3 Pressed");
}

void F4() 
{
   TV.fill(0);
   TV.println(2,2,"F4 Pressed");
}

void F5() 
{
   TV.fill(0);
   TV.println(2,2,"F5 Pressed");
}

void F6() 
{
   TV.fill(0);
   TV.println(2,2,"F6 Pressed"); 
}

void F7() 
{
   TV.fill(0);
   TV.println(2,2,"F7 Pressed");
}

void F8() 
{
   TV.fill(0);
   TV.println(2,2,"F8 Pressed");
}

void F9() 
{
   TV.fill(0);
   TV.println(2,2,"F9 Pressed");
}

void F10() 
{
   TV.fill(0);
   TV.println(2,2,"F10 Pressed");
}

void F11() 
{
   TV.fill(0);
   TV.println(2,2,"F11 Pressed");
}

void F12() 
{
   TV.fill(0);
   TV.println(2,2,"F12 Pressed");
}

//----------------------------------------------------------------------------------------------------
// Get PS/2 Keyboard input
//----------------------------------------------------------------------------------------------------
void loop() 
{
  getps2keys();
}

